package io.realm;


import android.annotation.TargetApi;
import android.os.Build;
import android.util.JsonReader;
import android.util.JsonToken;
import io.realm.RealmObjectSchema;
import io.realm.RealmSchema;
import io.realm.exceptions.RealmMigrationNeededException;
import io.realm.internal.ColumnInfo;
import io.realm.internal.LinkView;
import io.realm.internal.RealmObjectProxy;
import io.realm.internal.Row;
import io.realm.internal.SharedRealm;
import io.realm.internal.Table;
import io.realm.internal.TableOrView;
import io.realm.internal.android.JsonUtils;
import io.realm.log.RealmLog;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

public class InformationRealmProxy extends project.com.bachelor.datamodel.Information
    implements RealmObjectProxy, InformationRealmProxyInterface {

    static final class InformationColumnInfo extends ColumnInfo
        implements Cloneable {

        public long itemsIndex;

        InformationColumnInfo(String path, Table table) {
            final Map<String, Long> indicesMap = new HashMap<String, Long>(1);
            this.itemsIndex = getValidColumnIndex(path, table, "Information", "items");
            indicesMap.put("items", this.itemsIndex);

            setIndicesMap(indicesMap);
        }

        @Override
        public final void copyColumnInfoFrom(ColumnInfo other) {
            final InformationColumnInfo otherInfo = (InformationColumnInfo) other;
            this.itemsIndex = otherInfo.itemsIndex;

            setIndicesMap(otherInfo.getIndicesMap());
        }

        @Override
        public final InformationColumnInfo clone() {
            return (InformationColumnInfo) super.clone();
        }

    }
    private InformationColumnInfo columnInfo;
    private ProxyState<project.com.bachelor.datamodel.Information> proxyState;
    private RealmList<project.com.bachelor.datamodel.Items> itemsRealmList;
    private static final List<String> FIELD_NAMES;
    static {
        List<String> fieldNames = new ArrayList<String>();
        fieldNames.add("items");
        FIELD_NAMES = Collections.unmodifiableList(fieldNames);
    }

    InformationRealmProxy() {
        if (proxyState == null) {
            injectObjectContext();
        }
        proxyState.setConstructionFinished();
    }

    private void injectObjectContext() {
        final BaseRealm.RealmObjectContext context = BaseRealm.objectContext.get();
        this.columnInfo = (InformationColumnInfo) context.getColumnInfo();
        this.proxyState = new ProxyState<project.com.bachelor.datamodel.Information>(project.com.bachelor.datamodel.Information.class, this);
        proxyState.setRealm$realm(context.getRealm());
        proxyState.setRow$realm(context.getRow());
        proxyState.setAcceptDefaultValue$realm(context.getAcceptDefaultValue());
        proxyState.setExcludeFields$realm(context.getExcludeFields());
    }

    public RealmList<project.com.bachelor.datamodel.Items> realmGet$items() {
        if (proxyState == null) {
            // Called from model's constructor. Inject context.
            injectObjectContext();
        }

        proxyState.getRealm$realm().checkIfValid();
        // use the cached value if available
        if (itemsRealmList != null) {
            return itemsRealmList;
        } else {
            LinkView linkView = proxyState.getRow$realm().getLinkList(columnInfo.itemsIndex);
            itemsRealmList = new RealmList<project.com.bachelor.datamodel.Items>(project.com.bachelor.datamodel.Items.class, linkView, proxyState.getRealm$realm());
            return itemsRealmList;
        }
    }

    public void realmSet$items(RealmList<project.com.bachelor.datamodel.Items> value) {
        if (proxyState == null) {
            // Called from model's constructor. Inject context.
            injectObjectContext();
        }

        if (proxyState.isUnderConstruction()) {
            if (!proxyState.getAcceptDefaultValue$realm()) {
                return;
            }
            if (proxyState.getExcludeFields$realm().contains("items")) {
                return;
            }
            if (value != null && !value.isManaged()) {
                final Realm realm = (Realm) proxyState.getRealm$realm();
                final RealmList<project.com.bachelor.datamodel.Items> original = value;
                value = new RealmList<project.com.bachelor.datamodel.Items>();
                for (project.com.bachelor.datamodel.Items item : original) {
                    if (item == null || RealmObject.isManaged(item)) {
                        value.add(item);
                    } else {
                        value.add(realm.copyToRealm(item));
                    }
                }
            }
        }

        proxyState.getRealm$realm().checkIfValid();
        LinkView links = proxyState.getRow$realm().getLinkList(columnInfo.itemsIndex);
        links.clear();
        if (value == null) {
            return;
        }
        for (RealmModel linkedObject : (RealmList<? extends RealmModel>) value) {
            if (!(RealmObject.isManaged(linkedObject) && RealmObject.isValid(linkedObject))) {
                throw new IllegalArgumentException("Each element of 'value' must be a valid managed object.");
            }
            if (((RealmObjectProxy)linkedObject).realmGet$proxyState().getRealm$realm() != proxyState.getRealm$realm()) {
                throw new IllegalArgumentException("Each element of 'value' must belong to the same Realm.");
            }
            links.add(((RealmObjectProxy)linkedObject).realmGet$proxyState().getRow$realm().getIndex());
        }
    }

    public static RealmObjectSchema createRealmObjectSchema(RealmSchema realmSchema) {
        if (!realmSchema.contains("Information")) {
            RealmObjectSchema realmObjectSchema = realmSchema.create("Information");
            if (!realmSchema.contains("Items")) {
                ItemsRealmProxy.createRealmObjectSchema(realmSchema);
            }
            realmObjectSchema.add(new Property("items", RealmFieldType.LIST, realmSchema.get("Items")));
            return realmObjectSchema;
        }
        return realmSchema.get("Information");
    }

    public static Table initTable(SharedRealm sharedRealm) {
        if (!sharedRealm.hasTable("class_Information")) {
            Table table = sharedRealm.getTable("class_Information");
            if (!sharedRealm.hasTable("class_Items")) {
                ItemsRealmProxy.initTable(sharedRealm);
            }
            table.addColumnLink(RealmFieldType.LIST, "items", sharedRealm.getTable("class_Items"));
            table.setPrimaryKey("");
            return table;
        }
        return sharedRealm.getTable("class_Information");
    }

    public static InformationColumnInfo validateTable(SharedRealm sharedRealm, boolean allowExtraColumns) {
        if (sharedRealm.hasTable("class_Information")) {
            Table table = sharedRealm.getTable("class_Information");
            final long columnCount = table.getColumnCount();
            if (columnCount != 1) {
                if (columnCount < 1) {
                    throw new RealmMigrationNeededException(sharedRealm.getPath(), "Field count is less than expected - expected 1 but was " + columnCount);
                }
                if (allowExtraColumns) {
                    RealmLog.debug("Field count is more than expected - expected 1 but was %1$d", columnCount);
                } else {
                    throw new RealmMigrationNeededException(sharedRealm.getPath(), "Field count is more than expected - expected 1 but was " + columnCount);
                }
            }
            Map<String, RealmFieldType> columnTypes = new HashMap<String, RealmFieldType>();
            for (long i = 0; i < columnCount; i++) {
                columnTypes.put(table.getColumnName(i), table.getColumnType(i));
            }

            final InformationColumnInfo columnInfo = new InformationColumnInfo(sharedRealm.getPath(), table);

            if (table.hasPrimaryKey()) {
                throw new RealmMigrationNeededException(sharedRealm.getPath(), "Primary Key defined for field " + table.getColumnName(table.getPrimaryKey()) + " was removed.");
            }

            if (!columnTypes.containsKey("items")) {
                throw new RealmMigrationNeededException(sharedRealm.getPath(), "Missing field 'items'");
            }
            if (columnTypes.get("items") != RealmFieldType.LIST) {
                throw new RealmMigrationNeededException(sharedRealm.getPath(), "Invalid type 'Items' for field 'items'");
            }
            if (!sharedRealm.hasTable("class_Items")) {
                throw new RealmMigrationNeededException(sharedRealm.getPath(), "Missing class 'class_Items' for field 'items'");
            }
            Table table_0 = sharedRealm.getTable("class_Items");
            if (!table.getLinkTarget(columnInfo.itemsIndex).hasSameSchema(table_0)) {
                throw new RealmMigrationNeededException(sharedRealm.getPath(), "Invalid RealmList type for field 'items': '" + table.getLinkTarget(columnInfo.itemsIndex).getName() + "' expected - was '" + table_0.getName() + "'");
            }
            return columnInfo;
        } else {
            throw new RealmMigrationNeededException(sharedRealm.getPath(), "The 'Information' class is missing from the schema for this Realm.");
        }
    }

    public static String getTableName() {
        return "class_Information";
    }

    public static List<String> getFieldNames() {
        return FIELD_NAMES;
    }

    @SuppressWarnings("cast")
    public static project.com.bachelor.datamodel.Information createOrUpdateUsingJsonObject(Realm realm, JSONObject json, boolean update)
        throws JSONException {
        final List<String> excludeFields = new ArrayList<String>(1);
        if (json.has("items")) {
            excludeFields.add("items");
        }
        project.com.bachelor.datamodel.Information obj = realm.createObjectInternal(project.com.bachelor.datamodel.Information.class, true, excludeFields);
        if (json.has("items")) {
            if (json.isNull("items")) {
                ((InformationRealmProxyInterface) obj).realmSet$items(null);
            } else {
                ((InformationRealmProxyInterface) obj).realmGet$items().clear();
                JSONArray array = json.getJSONArray("items");
                for (int i = 0; i < array.length(); i++) {
                    project.com.bachelor.datamodel.Items item = ItemsRealmProxy.createOrUpdateUsingJsonObject(realm, array.getJSONObject(i), update);
                    ((InformationRealmProxyInterface) obj).realmGet$items().add(item);
                }
            }
        }
        return obj;
    }

    @SuppressWarnings("cast")
    @TargetApi(Build.VERSION_CODES.HONEYCOMB)
    public static project.com.bachelor.datamodel.Information createUsingJsonStream(Realm realm, JsonReader reader)
        throws IOException {
        project.com.bachelor.datamodel.Information obj = new project.com.bachelor.datamodel.Information();
        reader.beginObject();
        while (reader.hasNext()) {
            String name = reader.nextName();
            if (name.equals("items")) {
                if (reader.peek() == JsonToken.NULL) {
                    reader.skipValue();
                    ((InformationRealmProxyInterface) obj).realmSet$items(null);
                } else {
                    ((InformationRealmProxyInterface) obj).realmSet$items(new RealmList<project.com.bachelor.datamodel.Items>());
                    reader.beginArray();
                    while (reader.hasNext()) {
                        project.com.bachelor.datamodel.Items item = ItemsRealmProxy.createUsingJsonStream(realm, reader);
                        ((InformationRealmProxyInterface) obj).realmGet$items().add(item);
                    }
                    reader.endArray();
                }
            } else {
                reader.skipValue();
            }
        }
        reader.endObject();
        obj = realm.copyToRealm(obj);
        return obj;
    }

    public static project.com.bachelor.datamodel.Information copyOrUpdate(Realm realm, project.com.bachelor.datamodel.Information object, boolean update, Map<RealmModel,RealmObjectProxy> cache) {
        if (object instanceof RealmObjectProxy && ((RealmObjectProxy) object).realmGet$proxyState().getRealm$realm() != null && ((RealmObjectProxy) object).realmGet$proxyState().getRealm$realm().threadId != realm.threadId) {
            throw new IllegalArgumentException("Objects which belong to Realm instances in other threads cannot be copied into this Realm instance.");
        }
        if (object instanceof RealmObjectProxy && ((RealmObjectProxy)object).realmGet$proxyState().getRealm$realm() != null && ((RealmObjectProxy)object).realmGet$proxyState().getRealm$realm().getPath().equals(realm.getPath())) {
            return object;
        }
        final BaseRealm.RealmObjectContext objectContext = BaseRealm.objectContext.get();
        RealmObjectProxy cachedRealmObject = cache.get(object);
        if (cachedRealmObject != null) {
            return (project.com.bachelor.datamodel.Information) cachedRealmObject;
        } else {
            return copy(realm, object, update, cache);
        }
    }

    public static project.com.bachelor.datamodel.Information copy(Realm realm, project.com.bachelor.datamodel.Information newObject, boolean update, Map<RealmModel,RealmObjectProxy> cache) {
        RealmObjectProxy cachedRealmObject = cache.get(newObject);
        if (cachedRealmObject != null) {
            return (project.com.bachelor.datamodel.Information) cachedRealmObject;
        } else {
            // rejecting default values to avoid creating unexpected objects from RealmModel/RealmList fields.
            project.com.bachelor.datamodel.Information realmObject = realm.createObjectInternal(project.com.bachelor.datamodel.Information.class, false, Collections.<String>emptyList());
            cache.put(newObject, (RealmObjectProxy) realmObject);

            RealmList<project.com.bachelor.datamodel.Items> itemsList = ((InformationRealmProxyInterface) newObject).realmGet$items();
            if (itemsList != null) {
                RealmList<project.com.bachelor.datamodel.Items> itemsRealmList = ((InformationRealmProxyInterface) realmObject).realmGet$items();
                for (int i = 0; i < itemsList.size(); i++) {
                    project.com.bachelor.datamodel.Items itemsItem = itemsList.get(i);
                    project.com.bachelor.datamodel.Items cacheitems = (project.com.bachelor.datamodel.Items) cache.get(itemsItem);
                    if (cacheitems != null) {
                        itemsRealmList.add(cacheitems);
                    } else {
                        itemsRealmList.add(ItemsRealmProxy.copyOrUpdate(realm, itemsList.get(i), update, cache));
                    }
                }
            }

            return realmObject;
        }
    }

    public static long insert(Realm realm, project.com.bachelor.datamodel.Information object, Map<RealmModel,Long> cache) {
        if (object instanceof RealmObjectProxy && ((RealmObjectProxy)object).realmGet$proxyState().getRealm$realm() != null && ((RealmObjectProxy)object).realmGet$proxyState().getRealm$realm().getPath().equals(realm.getPath())) {
            return ((RealmObjectProxy)object).realmGet$proxyState().getRow$realm().getIndex();
        }
        Table table = realm.getTable(project.com.bachelor.datamodel.Information.class);
        long tableNativePtr = table.getNativeTablePointer();
        InformationColumnInfo columnInfo = (InformationColumnInfo) realm.schema.getColumnInfo(project.com.bachelor.datamodel.Information.class);
        long rowIndex = Table.nativeAddEmptyRow(tableNativePtr, 1);
        cache.put(object, rowIndex);

        RealmList<project.com.bachelor.datamodel.Items> itemsList = ((InformationRealmProxyInterface) object).realmGet$items();
        if (itemsList != null) {
            long itemsNativeLinkViewPtr = Table.nativeGetLinkView(tableNativePtr, columnInfo.itemsIndex, rowIndex);
            for (project.com.bachelor.datamodel.Items itemsItem : itemsList) {
                Long cacheItemIndexitems = cache.get(itemsItem);
                if (cacheItemIndexitems == null) {
                    cacheItemIndexitems = ItemsRealmProxy.insert(realm, itemsItem, cache);
                }
                LinkView.nativeAdd(itemsNativeLinkViewPtr, cacheItemIndexitems);
            }
        }

        return rowIndex;
    }

    public static void insert(Realm realm, Iterator<? extends RealmModel> objects, Map<RealmModel,Long> cache) {
        Table table = realm.getTable(project.com.bachelor.datamodel.Information.class);
        long tableNativePtr = table.getNativeTablePointer();
        InformationColumnInfo columnInfo = (InformationColumnInfo) realm.schema.getColumnInfo(project.com.bachelor.datamodel.Information.class);
        project.com.bachelor.datamodel.Information object = null;
        while (objects.hasNext()) {
            object = (project.com.bachelor.datamodel.Information) objects.next();
            if(!cache.containsKey(object)) {
                if (object instanceof RealmObjectProxy && ((RealmObjectProxy)object).realmGet$proxyState().getRealm$realm() != null && ((RealmObjectProxy)object).realmGet$proxyState().getRealm$realm().getPath().equals(realm.getPath())) {
                    cache.put(object, ((RealmObjectProxy)object).realmGet$proxyState().getRow$realm().getIndex());
                    continue;
                }
                long rowIndex = Table.nativeAddEmptyRow(tableNativePtr, 1);
                cache.put(object, rowIndex);

                RealmList<project.com.bachelor.datamodel.Items> itemsList = ((InformationRealmProxyInterface) object).realmGet$items();
                if (itemsList != null) {
                    long itemsNativeLinkViewPtr = Table.nativeGetLinkView(tableNativePtr, columnInfo.itemsIndex, rowIndex);
                    for (project.com.bachelor.datamodel.Items itemsItem : itemsList) {
                        Long cacheItemIndexitems = cache.get(itemsItem);
                        if (cacheItemIndexitems == null) {
                            cacheItemIndexitems = ItemsRealmProxy.insert(realm, itemsItem, cache);
                        }
                        LinkView.nativeAdd(itemsNativeLinkViewPtr, cacheItemIndexitems);
                    }
                }

            }
        }
    }

    public static long insertOrUpdate(Realm realm, project.com.bachelor.datamodel.Information object, Map<RealmModel,Long> cache) {
        if (object instanceof RealmObjectProxy && ((RealmObjectProxy)object).realmGet$proxyState().getRealm$realm() != null && ((RealmObjectProxy)object).realmGet$proxyState().getRealm$realm().getPath().equals(realm.getPath())) {
            return ((RealmObjectProxy)object).realmGet$proxyState().getRow$realm().getIndex();
        }
        Table table = realm.getTable(project.com.bachelor.datamodel.Information.class);
        long tableNativePtr = table.getNativeTablePointer();
        InformationColumnInfo columnInfo = (InformationColumnInfo) realm.schema.getColumnInfo(project.com.bachelor.datamodel.Information.class);
        long rowIndex = Table.nativeAddEmptyRow(tableNativePtr, 1);
        cache.put(object, rowIndex);

        long itemsNativeLinkViewPtr = Table.nativeGetLinkView(tableNativePtr, columnInfo.itemsIndex, rowIndex);
        LinkView.nativeClear(itemsNativeLinkViewPtr);
        RealmList<project.com.bachelor.datamodel.Items> itemsList = ((InformationRealmProxyInterface) object).realmGet$items();
        if (itemsList != null) {
            for (project.com.bachelor.datamodel.Items itemsItem : itemsList) {
                Long cacheItemIndexitems = cache.get(itemsItem);
                if (cacheItemIndexitems == null) {
                    cacheItemIndexitems = ItemsRealmProxy.insertOrUpdate(realm, itemsItem, cache);
                }
                LinkView.nativeAdd(itemsNativeLinkViewPtr, cacheItemIndexitems);
            }
        }

        return rowIndex;
    }

    public static void insertOrUpdate(Realm realm, Iterator<? extends RealmModel> objects, Map<RealmModel,Long> cache) {
        Table table = realm.getTable(project.com.bachelor.datamodel.Information.class);
        long tableNativePtr = table.getNativeTablePointer();
        InformationColumnInfo columnInfo = (InformationColumnInfo) realm.schema.getColumnInfo(project.com.bachelor.datamodel.Information.class);
        project.com.bachelor.datamodel.Information object = null;
        while (objects.hasNext()) {
            object = (project.com.bachelor.datamodel.Information) objects.next();
            if(!cache.containsKey(object)) {
                if (object instanceof RealmObjectProxy && ((RealmObjectProxy)object).realmGet$proxyState().getRealm$realm() != null && ((RealmObjectProxy)object).realmGet$proxyState().getRealm$realm().getPath().equals(realm.getPath())) {
                    cache.put(object, ((RealmObjectProxy)object).realmGet$proxyState().getRow$realm().getIndex());
                    continue;
                }
                long rowIndex = Table.nativeAddEmptyRow(tableNativePtr, 1);
                cache.put(object, rowIndex);

                long itemsNativeLinkViewPtr = Table.nativeGetLinkView(tableNativePtr, columnInfo.itemsIndex, rowIndex);
                LinkView.nativeClear(itemsNativeLinkViewPtr);
                RealmList<project.com.bachelor.datamodel.Items> itemsList = ((InformationRealmProxyInterface) object).realmGet$items();
                if (itemsList != null) {
                    for (project.com.bachelor.datamodel.Items itemsItem : itemsList) {
                        Long cacheItemIndexitems = cache.get(itemsItem);
                        if (cacheItemIndexitems == null) {
                            cacheItemIndexitems = ItemsRealmProxy.insertOrUpdate(realm, itemsItem, cache);
                        }
                        LinkView.nativeAdd(itemsNativeLinkViewPtr, cacheItemIndexitems);
                    }
                }

            }
        }
    }

    public static project.com.bachelor.datamodel.Information createDetachedCopy(project.com.bachelor.datamodel.Information realmObject, int currentDepth, int maxDepth, Map<RealmModel, CacheData<RealmModel>> cache) {
        if (currentDepth > maxDepth || realmObject == null) {
            return null;
        }
        CacheData<RealmModel> cachedObject = cache.get(realmObject);
        project.com.bachelor.datamodel.Information unmanagedObject;
        if (cachedObject != null) {
            // Reuse cached object or recreate it because it was encountered at a lower depth.
            if (currentDepth >= cachedObject.minDepth) {
                return (project.com.bachelor.datamodel.Information)cachedObject.object;
            } else {
                unmanagedObject = (project.com.bachelor.datamodel.Information)cachedObject.object;
                cachedObject.minDepth = currentDepth;
            }
        } else {
            unmanagedObject = new project.com.bachelor.datamodel.Information();
            cache.put(realmObject, new RealmObjectProxy.CacheData<RealmModel>(currentDepth, unmanagedObject));
        }

        // Deep copy of items
        if (currentDepth == maxDepth) {
            ((InformationRealmProxyInterface) unmanagedObject).realmSet$items(null);
        } else {
            RealmList<project.com.bachelor.datamodel.Items> manageditemsList = ((InformationRealmProxyInterface) realmObject).realmGet$items();
            RealmList<project.com.bachelor.datamodel.Items> unmanageditemsList = new RealmList<project.com.bachelor.datamodel.Items>();
            ((InformationRealmProxyInterface) unmanagedObject).realmSet$items(unmanageditemsList);
            int nextDepth = currentDepth + 1;
            int size = manageditemsList.size();
            for (int i = 0; i < size; i++) {
                project.com.bachelor.datamodel.Items item = ItemsRealmProxy.createDetachedCopy(manageditemsList.get(i), nextDepth, maxDepth, cache);
                unmanageditemsList.add(item);
            }
        }
        return unmanagedObject;
    }

    @Override
    public String toString() {
        if (!RealmObject.isValid(this)) {
            return "Invalid object";
        }
        StringBuilder stringBuilder = new StringBuilder("Information = [");
        stringBuilder.append("{items:");
        stringBuilder.append("RealmList<Items>[").append(realmGet$items().size()).append("]");
        stringBuilder.append("}");
        stringBuilder.append("]");
        return stringBuilder.toString();
    }

    @Override
    public ProxyState realmGet$proxyState() {
        return proxyState;
    }

    @Override
    public int hashCode() {
        String realmName = proxyState.getRealm$realm().getPath();
        String tableName = proxyState.getRow$realm().getTable().getName();
        long rowIndex = proxyState.getRow$realm().getIndex();

        int result = 17;
        result = 31 * result + ((realmName != null) ? realmName.hashCode() : 0);
        result = 31 * result + ((tableName != null) ? tableName.hashCode() : 0);
        result = 31 * result + (int) (rowIndex ^ (rowIndex >>> 32));
        return result;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        InformationRealmProxy aInformation = (InformationRealmProxy)o;

        String path = proxyState.getRealm$realm().getPath();
        String otherPath = aInformation.proxyState.getRealm$realm().getPath();
        if (path != null ? !path.equals(otherPath) : otherPath != null) return false;

        String tableName = proxyState.getRow$realm().getTable().getName();
        String otherTableName = aInformation.proxyState.getRow$realm().getTable().getName();
        if (tableName != null ? !tableName.equals(otherTableName) : otherTableName != null) return false;

        if (proxyState.getRow$realm().getIndex() != aInformation.proxyState.getRow$realm().getIndex()) return false;

        return true;
    }

}
