package project.com.bachelor.model;

import java.util.ArrayList;

import project.com.bachelor.connection.RetrofitHelper;
import project.com.bachelor.connection.UrlPath;
import project.com.bachelor.datamodel.Information;
import project.com.bachelor.utils.CustomLog;
import io.realm.Realm;
import io.realm.RealmResults;
import rx.Observer;
import rx.android.schedulers.AndroidSchedulers;
import rx.schedulers.Schedulers;

/**
 * Created by ReneUhrin on 17.12.16.
 * Email: reneuhrin@gmail.com
 */

public class ApiHandler {

    private static final String TAG = ApiHandler.class.getSimpleName();

    private Realm realm;

    /**
     * Very simple interface that represents api call result callback
     *
     */
    public interface ResultCallback<T> {
        void onSuccess(T response);

        void onAppStart(ArrayList<?> data);

        void onError(String reason);
    }

    public void getInformationDB(ResultCallback<Information> callback) {
        realm = Realm.getDefaultInstance();
        RealmResults<Information> moviesDB = realm.where(Information.class).findAll();

        // Its better convert RealmResult to ArrayList because Realm is external library and I don't want to have it in whole project.
        ArrayList<Information> list = new ArrayList<>();
        list.addAll(moviesDB.subList(0, moviesDB.size()));

        callback.onAppStart(list);
    }

    public void getInformation(int page, final ResultCallback<Information> callback) {

        // in this project we can do it like this. But its not problem every argument do like PAGE
        final String FILTER = "withbody";
        final String FROM_DATE = "1459468800";
        final String TO_DATE = "1461974400";
        final String ORDER = "desc";
        final String SORT = "creation";
        final String SITE = "cooking";
        final String PAGE_SIZE = "5";

        final UrlPath service = RetrofitHelper.getInstance().getUrlPath();
        realm = Realm.getDefaultInstance();

        service.getData(FILTER, FROM_DATE, TO_DATE, ORDER, SORT, SITE, PAGE_SIZE, String.valueOf(page))
                .subscribeOn(Schedulers.newThread())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(new Observer<Information>() {

                    @Override
                    public void onCompleted() {

                    }

                    @Override
                    public void onError(Throwable e) {
                        // Called when the observable encounters an error
                        CustomLog.d(TAG, ">>>> onError gets called : " + e.getMessage());
                        callback.onError(e.getMessage());
                    }

                    @Override
                    public void onNext(Information information) {
                        CustomLog.d(TAG, ">>>> Success, new data from server");

                        realm.beginTransaction();
                        // Copy the objects to realm. The list still contains detached objects.
                        // If you want to use the live objects, you have to use the return value
                        // of this call.
                        realm.copyToRealm(information);
                        realm.commitTransaction();

                        callback.onSuccess(information);

                    }
                });
    }
}
