package project.com.bachelor.presenter;

import java.util.ArrayList;

import project.com.bachelor.datamodel.Information;
import project.com.bachelor.model.ApiHandler;
import project.com.bachelor.view.IMainView;

/**
 * Created by ReneUhrin on 17.12.16.
 * Email: reneuhrin@gmail.com
 */

public class MainPresenter implements ApiHandler.ResultCallback<Information> {

    private static final String TAG = MainPresenter.class.getSimpleName();
    private IMainView mView;
    private String mError;
    private boolean mCallingfinished;
    private final ApiHandler mApiHandler;
    private Information mBasePojoResponse;
    private ArrayList<?> mInformationDBResponse;

    public MainPresenter() {
        mApiHandler = new ApiHandler();
    }

    public void onViewAttached(IMainView view) {
        mView = view;
        updateView();
    }

    public void onViewDetached() {
        mView = null;
    }

    private IMainView getView() {
        return mView;
    }

    public void getInformation(int page) {
        mError = null;
        mCallingfinished = false;
        getView().showProgress(true);
        mApiHandler.getInformation(page, this);
    }

    public void appFirstStart() {
        mError = null;
        mCallingfinished = false;
        getView().showProgress(true);
        mApiHandler.getInformationDB(this);
    }

    private void updateView() {
        if (mError != null) {
            getView().onGeneralError(mError);
        }

        if (mCallingfinished) {
            if (mBasePojoResponse != null) {
                getView().onNotifyArray(mBasePojoResponse);
                mBasePojoResponse = null;
            } else if (mInformationDBResponse != null) {
                getView().onAppStart(mInformationDBResponse);
                mInformationDBResponse = null;
            }
        }
    }


    @Override
    public void onSuccess(Information response) {
        mBasePojoResponse = response;
        mCallingfinished = true;
        getView().showProgress(false);
        updateView();
    }

    @Override
    public void onAppStart(ArrayList<?> data) {
        mInformationDBResponse = data;
        mCallingfinished = true;
        getView().showProgress(false);
        updateView();
    }

    @Override
    public void onError(String reason) {
        mError = reason;
        getView().showProgress(false);
        updateView();
    }
}
