package project.com.bachelor.view;

import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ProgressBar;
import android.widget.RelativeLayout;

import java.util.ArrayList;

import project.com.bachelor.App;
import project.com.bachelor.R;
import project.com.bachelor.datamodel.Items;
import project.com.bachelor.datamodel.Information;
import project.com.bachelor.presenter.MainPresenter;
import project.com.bachelor.utils.ConstantsString;
import project.com.bachelor.utils.CustomToast;
import project.com.bachelor.utils.EndlessRecyclerViewScrollListener;
import project.com.bachelor.utils.IClickRecyclerView;
import project.com.bachelor.utils.UtilsPreferences;
import project.com.bachelor.view.adapter.LoadDataRecyclerViewAdapter;

public class ListFragment extends Fragment implements IMainView {

    private MainPresenter mPresenter;
    private RecyclerView mRvList;
    private RecyclerView.Adapter mRvAdapter;
    private ProgressBar mPbLoading;
    private ArrayList<Items> mItemsArrayList = new ArrayList<>();
    private LinearLayoutManager mLayoutManager;
    private RelativeLayout mRlErrorWrapper;
    private Button mBAgain;
    private IClickRecyclerView mIClickRecyclerView;

    private boolean mIsErrorWrapperVisible = false;
    private boolean mFirstStart = true;

    public ListFragment() {
        // Required empty public constructor
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);

//        outState.putBoolean(ConstantsString.ARG_IS_ERROR_HANDLER_VISIBLE, mRlErrorWrapper.getVisibility() == View.VISIBLE);
//        outState.putBoolean(ConstantsString.ARG_FIRST_START, true);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View view = inflater.inflate(R.layout.fragment_list, container, false);

        if (mPresenter == null) {//first time creating activity = create presenter
            // if it is create in onCreateView it has to be destroyed in onDestroyView
            mPresenter = new MainPresenter();
        }

        getPresenter().onViewAttached(this);

        Toolbar toolbar = (Toolbar) view.findViewById(R.id.toolbarCustomToolbar);
        ((MainActivity) getActivity()).setSupportActionBar(toolbar);

        ((MainActivity) getActivity()).getSupportActionBar().setDisplayShowTitleEnabled(true);
        ((MainActivity) getActivity()).getSupportActionBar().setDisplayHomeAsUpEnabled(false);
        ((MainActivity) getActivity()).getSupportActionBar().setTitle(App.getContext().getString(R.string.app_name));

        toolbar.setNavigationOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                getActivity().onBackPressed();
            }
        });

        mRvList = (RecyclerView) view.findViewById(R.id.recyclerViewListActivityMain);
        mPbLoading = (ProgressBar) view.findViewById(R.id.progressBarLoadingActivityMain);
        mRlErrorWrapper = (RelativeLayout) view.findViewById(R.id.relativeLayoutErrorWrapperActivityMain);
        mBAgain = (Button) view.findViewById(R.id.buttonAgainActivityMain);

        if (savedInstanceState != null) {
            mIsErrorWrapperVisible = savedInstanceState.getBoolean(ConstantsString.ARG_IS_ERROR_HANDLER_VISIBLE);

            if (mIsErrorWrapperVisible) {
                mRlErrorWrapper.setVisibility(View.VISIBLE);
            } else {
                mRlErrorWrapper.setVisibility(View.GONE);
            }

            mFirstStart = savedInstanceState.getBoolean(ConstantsString.ARG_FIRST_START, true);
        }

        mBAgain.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                mPresenter.getInformation(UtilsPreferences.getPage());
            }
        });

        mIClickRecyclerView = new IClickRecyclerView() {
            @Override
            public void onClickListener(View view, int position) {

                // The user selected the headline of an article from the HeadlinesFragment
                // Capture the article fragment from the activity layout
                DetailFragment detailFragment = (DetailFragment) getFragmentManager().findFragmentById(R.id.detail_fragment);

                if (detailFragment == null) {
                    // If the frag is not available, we're in the one-pane layout and must swap frags...
                    // Create fragment and give it an argument for the selected article
                    UtilsPreferences.replaceFragment(R.id.activity_main, DetailFragment.newInstance(mItemsArrayList.get(position).getTitle(), mItemsArrayList.get(position).getBody(), false), getFragmentManager());
                } else {
                    // If article frag is available, we're in two-pane layout...
                    detailFragment.setInformation(mItemsArrayList.get(position));
                }
            }
        };

        // use this setting to improve performance if you know that changes
        // in content do not change the layout size of the RecyclerView
        mRvList.setHasFixedSize(true);

        // use a linear layout manager
        mLayoutManager = new LinearLayoutManager(getContext());
        mRvList.setLayoutManager(mLayoutManager);
        mRvAdapter = new LoadDataRecyclerViewAdapter(mItemsArrayList, mIClickRecyclerView);
        mRvList.setAdapter(mRvAdapter);
        mRvList.addOnScrollListener(new EndlessRecyclerViewScrollListener(mLayoutManager) {
            @Override
            public void onLoadMore(int page, int totalItemsCount) {
                mPresenter.getInformation(page);
            }
        });

        if (mFirstStart) {
            mFirstStart = false;
            mPresenter.appFirstStart();
        }


        return view;
    }

    @Override
    public void showProgress(boolean show) {
        if (show) {
            mPbLoading.setVisibility(View.VISIBLE);
        } else {
            mPbLoading.setVisibility(View.GONE);
        }
    }

    @Override
    public void onGeneralError(String error) {
        CustomToast.getLongToast(getContext(), error);
        mRlErrorWrapper.setVisibility(View.VISIBLE);
    }

    @Override
    public void onNotifyArray(Information information) {
        mRlErrorWrapper.setVisibility(View.GONE);

        for (int i = 0; i < information.getItems().size(); i++) {
            mItemsArrayList.add(information.getItems().get(i));
        }

        mRvAdapter.notifyDataSetChanged();
    }

    @Override
    public void onAppStart(ArrayList<?> data) {

        ArrayList<Information> dataInformation = new ArrayList<>();

        ArrayList<Information> informationRealmResult = (ArrayList<Information>) data;
        dataInformation.addAll(informationRealmResult.subList(0, informationRealmResult.size()));

        for (int i = 0; i < dataInformation.size(); i++) {
            for (int j = 0; j < dataInformation.get(i).getItems().size(); j++) {
                mItemsArrayList.add(dataInformation.get(i).getItems().get(j));
            }
        }

        if (mItemsArrayList.size() == 0) {
            mPresenter.getInformation(UtilsPreferences.getPage());
        } else {
            mRvAdapter.notifyDataSetChanged();
        }
    }

    private MainPresenter getPresenter() {
        return mPresenter;
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();

        getPresenter().onViewDetached();
        if (getActivity().isFinishing()) {//activity finishing, release reference to presenter
            mPresenter = null;
        }
    }
}
