package project.com.bachelor.view.adapter;

import android.support.v7.widget.RecyclerView;
import android.text.Html;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.TextView;

import java.util.ArrayList;

import project.com.bachelor.R;
import project.com.bachelor.datamodel.Items;
import project.com.bachelor.utils.IClickRecyclerView;

/**
 * Created by ReneUhrin on 17.12.16.
 * Email: reneuhrin@gmail.com
 */

public class LoadDataRecyclerViewAdapter extends RecyclerView.Adapter<LoadDataRecyclerViewAdapter.ViewHolder> {

    private static final String TAG = LoadDataRecyclerViewAdapter.class.getSimpleName();

    private ArrayList<Items> mDataset;
    private IClickRecyclerView mIClickRecyclerView;

    // Provide a reference to the views for each data item
    // Complex data items may need more than one view per item, and
    // you provide access to all the views for a data item in a view holder
    public static class ViewHolder extends RecyclerView.ViewHolder {

        private TextView mTvTitle;
        private LinearLayout mLlTitleWrapper;

        private ViewHolder(View v) {
            super(v);
            mTvTitle = (TextView) v.findViewById(R.id.textViewTitleItemRecyclerView);
            mLlTitleWrapper = (LinearLayout) v.findViewById(R.id.linearLayoutTitleWrapperItemRecyclerView);

        }
    }

    public LoadDataRecyclerViewAdapter(ArrayList<Items> myDataset, IClickRecyclerView iClickRecyclerView) {
        mDataset = myDataset;
        mIClickRecyclerView = iClickRecyclerView;
    }

    // Create new views (invoked by the layout manager)
    @Override
    public LoadDataRecyclerViewAdapter.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        // create a new view
        View v = LayoutInflater.from(parent.getContext())
                .inflate(R.layout.item_recycler_view, parent, false);

        return new ViewHolder(v);
    }

    // Replace the contents of a view (invoked by the layout manager)
    @Override
    public void onBindViewHolder(final ViewHolder holder, int position) {
        holder.mTvTitle.setText(Html.fromHtml(mDataset.get(position).getTitle()));
        holder.mLlTitleWrapper.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                mIClickRecyclerView.onClickListener(holder.mTvTitle, holder.getAdapterPosition());
            }
        });
    }


    // Return the size of your dataset (invoked by the layout manager)
    @Override
    public int getItemCount() {
        return mDataset.size();
    }
}
